#include <owl.h>         // CTLFNT adds a Colored Text Custom
#include <bwcc.h>        // Control that interfaces with the
#include <custcntl.h>    // The Resource Workshop as well as
#include <control.h>     // functions as a stand alone control
#include <static.h>
#include <edit.h>
#include <bstatic.h>
#include "ctlfnt.h"
#include <commdlg.h>    // for Win 3.1 Common Dialogs

PTModule CtlFntModule;
//=======================================================================//
_CLASSDEF(TFontDialog)               // Class to encapsulate
class TFontDialog : public TDialog   // the Windows 3.1
  {                                  // Font Common Dialog
  public:
    TFontDialog(PTWindowsObject AParent, // Constructor
		 LOGFONT far * lfNewFont, DWORD * dwNewColor,
		 LPSTR lpName, PTModule AModule);
    virtual BOOL Create();            // Create for non-modal Creation
    virtual int Execute();            // Execute for modal Execution
    #pragma argsused                  // Ignore ARetValue non use
    virtual void CloseWindow(int ARetValue){}; // trap destruction
  protected:
    DWORD * dwColor;                 // Font Color
    CHOOSEFONT fcdlg;                // Font Common Dialog Structure
  };
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
TFontDialog::TFontDialog(PTWindowsObject AParent, LOGFONT far * lfNewFont,
			 DWORD * dwNewColor, LPSTR AName,
			 PTModule AModule = NULL)
	     :TDialog(AParent,AName,AModule) // Constructor calls base
  {
  dwColor = dwNewColor;     // equate color address to allow updateing
  fcdlg.hDC            = HDC(NULL);             // used for printer fonts
  fcdlg.lStructSize    = sizeof(CHOOSEFONT);    // initialize ccdl struct
  fcdlg.hwndOwner      = GetFocus();            // parent has the focus
  fcdlg.lpLogFont      = lfNewFont;             // LOGFONT structure
  fcdlg.Flags          = CF_SCREENFONTS | CF_EFFECTS; // no printer fonts
  fcdlg.rgbColors      = * dwNewColor;          // Font Color selection
  fcdlg.lCustData      = 0L;                    // Use Default Data
  fcdlg.lpfnHook       = (FARPROC)NULL;         // No Message trapping
  fcdlg.lpTemplateName = (LPSTR)NULL;           // Use Default dialog
  fcdlg.hInstance      = (HANDLE) NULL;         // Not Used
  fcdlg.lpszStyle      = (LPSTR)NULL;           // Used with CF_USESTYLE
  fcdlg.nFontType      = SCREEN_FONTTYPE;       // Screen Font
  fcdlg.nSizeMin       = 0;                     // Used with CF_LIMITSIZE
  fcdlg.nSizeMax       = 0;                     // Used with CF_LIMITSIZE
  };
BOOL TFontDialog::Create()                      // Called by MakeWindow
  {
  fcdlg.hwndOwner = NULL;                       // No parent for non-Model
  ChooseFont(&fcdlg);                           // fcdlg function call
  *dwColor = fcdlg.rgbColors;                   // update color variable
  return 1;                                     // Success
  }
int TFontDialog::Execute()                      // Called by CreateDialog
  {
  ChooseFont(&fcdlg);                           // fcdlg function call
  *dwColor = fcdlg.rgbColors;                   // update color variable
  return 1;                                     // Success
  }
//=======================================================================//
char szClassName[]  = "FontText";   // class for new custom control
//=====================================================================
#pragma argsused  // TFntText methods prototype in header file
TFntText::TFntText(PTWindowsObject AParent, int AnId, LPSTR ATitle,
		     int X, int Y, int W, int H, WORD  ATextLen,
		     DWORD * dwColor, LOGFONT far * lfFont, PTModule AModule)
		    :TStatic (AParent, AnId, ATitle,X,Y,W,H,
			      ATextLen,AModule)
  {
  dwFontColor = * dwColor;
  lfDisplay   = * lfFont;
  };
//------------------------------------------------------------------------
TFntText::TFntText(PTWindowsObject AParent, int ResourceId,
		       DWORD * dwColor, LOGFONT far * lfFont,
		       PTModule AModule)
		      :TStatic(AParent, ResourceId, 1, AModule)
  {
  dwFontColor = * dwColor;
  lfDisplay   = * lfFont;
  };
//------------------------------------------------------------------------
LPSTR TFntText::GetClassName()
      {
      return szClassName;
      }
//------------------------------------------------------------------------
void TFntText::WMPaint(RTMessage Msg)
  {
  HDC hDC     = GetDC(HWindow);                 // grab the Device Contect
  HFONT hFont = CreateFontIndirect(&lfDisplay);        // create a new font
  HFONT hFontOld = (HFONT)SelectObject(hDC, hFont); // select it into display
  SetTextColor(hDC,dwFontColor);                    // set the text Color
  SetBkMode (hDC, TRANSPARENT);                     // Clear Text Background
  char szBuff[256];                                 // Buffer for text
  GetText(szBuff,255);                              // Get from title
  TextOut(hDC,0,0,szBuff,lstrlen(szBuff));          // Output string
  SelectObject(hDC,hFontOld);                       // restore old font
  DeleteObject(hFont);                              // delete font handle
  ReleaseDC(HWindow,hDC);                           // release the DC
  ValidateRect(HWindow,NULL);                       // Mark window painted
  Msg.Result = 0;                                   // Success
  }
//------------------------------------------------------------------------
LONG FAR PASCAL _export FntTextWndProc(HWND HWindow, // Main process
					WORD wMsg,   // registered
					WORD wParam, // in following
					LONG lParam) // function
  {
  if (wMsg == WM_CREATE)    // if Called by non-OWL application (RW)
    {
    LOGFONT lfFont;
    DWORD dwColor;          // default text color

    CREATESTRUCT  *pCS;
    pCS = (CREATESTRUCT *) lParam;
    BYTE * pExtra = (BYTE *)pCS->lpCreateParams;

    if (pExtra)
      {
      char myString[255];
      int i; int t;
      for (t=0,i=0;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfHeight = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfWidth = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfEscapement = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfOrientation = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfWeight = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfItalic = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfUnderline = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfStrikeOut = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfCharSet = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfOutPrecision = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfClipPrecision = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfQuality = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lfFont.lfPitchAndFamily = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      lstrcpy (lfFont.lfFaceName,myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      int nRed = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      int nGreen = atoi(myString);
      for (t=0,i++;(pExtra[i]!='~');t++,i++) myString[t] = pExtra[i];
      myString[t] = NULL;
      int nBlue = atoi(myString);
      dwColor = RGB(nRed,nGreen,nBlue);
      }
    else
      {                                        // default Text and Font
      dwColor = RGB(0,0,25);                   // Blue Text
      lfFont.lfHeight = 36;                    // Fill structure with
      lfFont.lfWidth  = 14;                    // Information for large
      lfFont.lfEscapement = 0;                 // Times Font
      lfFont.lfOrientation = 0;
      lfFont.lfWeight = 700;
      lfFont.lfItalic = 0;
      lfFont.lfUnderline = 0;
      lfFont.lfStrikeOut = 0;
      lfFont.lfCharSet = ANSI_CHARSET;
      lfFont.lfQuality = DEFAULT_QUALITY;
      lfFont.lfPitchAndFamily = VARIABLE_PITCH | FF_DONTCARE;
      lstrcpy(lfFont.lfFaceName, "Times New Roman");
      }
      // pass the data in the Constructor
    PTFntText PFntText =  // Create an instance
      new TFntText(CtlFntModule->GetParentObject(GetParent(HWindow)),
		     GetWindowWord(HWindow, GWW_ID),&dwColor,&lfFont,
		     CtlFntModule);
    CtlFntModule->MakeWindow(PFntText); // hook module to a window
    }
  return DefWindowProc(HWindow, wMsg, wParam, lParam); // allow Windows
  }          // to handle messages prior to OWL hooking into the window
//------------------------------------------------------------------------
BOOL RegisterFntTextClass(HINSTANCE hInstance)  // Called by Libmain to
{                                           // register the Window Class
  WNDCLASS  wc;
  wc.style         = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_GLOBALCLASS;
  wc.lpfnWndProc   = (WNDPROC)FntTextWndProc;  // Registers Proc used by Class
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(NULL_BRUSH);    //(Clear Background);
  wc.lpszMenuName  = NULL;
  wc.lpszClassName   = szClassName;
  return RegisterClass(&wc);
  }
//------------------------------------------------------------------------
_CLASSDEF(TFntTextDlg)              // Workshop Functions
class TFntTextDlg : public TDialog  // Dialog Class used by Workshop when
  {                                 // The Control is double clicked
  protected:
    LPFNSTRTOID    StrToId;         // Pointers to items in workshop
    LPFNIDTOSTR    IdToStr;         // That are used in dialog
    LPRWCTLSTYLE     Style;
    PTEdit         pTitle;          // Control pointers
    PTEdit         pIdText;
    PTBStatic      pIdNum;
    char           szBuffer[80];    // Line Buffer
    WORD           wType;           // Control Type
    DWORD dwFontColor;                              // font color
    LOGFONT lfFont;                                 // font structure
  public:
    TFntTextDlg(PTWindowsObject AParent, LPSTR AName,
	      LPFNSTRTOID PStrToId, LPFNIDTOSTR PIdToStr,
	      LPRWCTLSTYLE  PStyle, PTModule AModule = NULL);
    void SetupWindow();
    virtual void FontButton(RTMessage Msg) // button handler
      = [ID_FIRST + IDD_FONT];
    void Ok(RTMessage Msg) = [ID_FIRST + IDOK];
  };
//------------------------------------------------------------------------
TFntTextDlg::TFntTextDlg(PTWindowsObject AParent, LPSTR AName,
		     LPFNSTRTOID PStrToId, LPFNIDTOSTR PIdToStr,
		     LPRWCTLSTYLE  PStyle, PTModule AModule)
	  :TDialog(AParent, AName, AModule),    // Base Class
	   StrToId(PStrToId),                   // Three functions
	   IdToStr(PIdToStr),                   // in Worshop that
	   Style(PStyle)                        // are used
  {
  pTitle      = new TEdit(this,      IDD_TITLE, CTLTITLE, CtlFntModule);
  pIdText     = new TEdit(this,      IDD_IDTEXT, 255, CtlFntModule);
  pIdNum      = new TBStatic(this,   IDD_IDNUM,   18, CtlFntModule);
  }
//---------------------------------------------------------------------
void TFntTextDlg::SetupWindow()
  {
  char  buf[256];
  TDialog::SetupWindow();             // call base class
  pTitle->SetText(Style->szTitle);    // and initialize controls
  IdToStr(Style->wId, buf, 255);      // Use Workshop to get ID
  pIdText->SetText(buf);              // String and store it
  itoa(Style->wId, buf, 10);          // Translate it to a Number
  pIdNum->SetText(buf);               // and Store Number
  wType = 0xFFFFL & LOWORD(Style->dwStyle); // Save control Type
  }
//----------------------------------------------------------------------
void TFntTextDlg:: FontButton(RTMessage)     // button activates
  {
  CtlFntModule->ExecDialog(new TFontDialog(
			    CtlFntModule->GetParentObject(HWindow),
			    &lfFont,&dwFontColor,"Font_Dialog",
			    CtlFntModule));
  }
//----------------------------------------------------------------------
#pragma argsused
void TFntTextDlg::Ok(RTMessage Msg)      // OK Button pressed
  {
  char  szBuf[256];                   // Buffer for ID string
  LONG  lRetId;                       // Id by Value
  pIdText->GetText(szBuf, 255);       // Get the Text for the Control ID
  lRetId = StrToId(szBuf);            // Convert the define to its number
  if (LOWORD(lRetId) == 0)            // if the number is blank
    {
    pIdText->SetSelection(0, 32767);  // Select a default Number
    SetFocus(pIdText->HWindow);       // Set the focus to the Control
    Msg.Result = (LONG)TRUE;          // return(TRUE);
    return;                           // Return to Dialog for more input
    }
  Style->wId = HIWORD(lRetId);             // not blank - so set the wId
  pTitle->GetText(szBuf, CTLTITLE);        // Get the Title from control
  lstrcpy(Style->szTitle, szBuf);           // Copy to structure
  Style->dwStyle = WS_CHILD | WS_VISIBLE | wType ;

  wsprintf(szBuf,"%d~%d~%d~%d~%d~%d~%d~%d~%d~%d~%d~%d~%d~%s~%d~%d~%d~",
	  lfFont.lfHeight,       lfFont.lfWidth,         lfFont.lfEscapement,
	  lfFont.lfOrientation,  lfFont.lfWeight,        lfFont.lfItalic,
	  lfFont.lfUnderline,    lfFont.lfStrikeOut,     lfFont.lfCharSet,
	  lfFont.lfOutPrecision, lfFont.lfClipPrecision, lfFont.lfQuality,
	  lfFont.lfPitchAndFamily, lfFont.lfFaceName,
	  LOBYTE(LOWORD(dwFontColor)),HIBYTE(LOWORD(dwFontColor)),
	  LOBYTE(HIWORD(dwFontColor)) );

  int i;
  for (i=0;i<lstrlen(szBuf);i++)
    Style->CtlData[i] = szBuf[i];
  Style->CtlDataSize = lstrlen(szBuf);

  CloseWindow(TRUE);                       // Close Window and update
  }                                        // Workshop Structure
//-------------------------------------------------------------------
#pragma argsused                  // displays dialog box in RW to allow
BOOL FAR PASCAL _export FntTextStyle(HWND HWindow,         // user to
				      HANDLE hCtlStyle,     // change
				      LPFNSTRTOID StrToId,  // control
				      LPFNIDTOSTR IdToStr)  // attributes
{
  PTFntTextDlg    PDlg;
  LPCTLSTYLE    PStyle  = (LPCTLSTYLE) GlobalLock(hCtlStyle);
  BOOL          fRetval = FALSE;
  if (PStyle)                // If Style locked OK
    {                        // then Display dialog box
      PDlg = new TFntTextDlg(CtlFntModule->GetParentObject(HWindow),
			 (LPSTR)MAKEINTRESOURCE(CTLFNTDLG),
			 (LPFNSTRTOID)StrToId,
			 (LPFNIDTOSTR)IdToStr,
			 (LPRWCTLSTYLE)PStyle,
			 CtlFntModule);
    fRetval = (IDOK == CtlFntModule->ExecDialog(PDlg) );
    GlobalUnlock(hCtlStyle);  // cleanup
  }
  return fRetval;             // return Flag to RW
}
//-------------------------------------------------------------------
#pragma argsused                                     // Called by RW to
WORD FAR PASCAL _export FntTextFlags(DWORD dwStyle,  // translate style
				      LPSTR szBuf,   // bits to text
				      WORD wbufLen)  // for clarity
  {
  wsprintf(szBuf,(LPSTR)"0");  // No Style bits so use
  return lstrlen(szBuf);       // the Character for zero
  }
//-------------------------------------------------------------------
HANDLE FAR PASCAL _export FntTextInfo(void)   // Info function for RW
{                                             // Default Control Settings
  HANDLE hInfo = GlobalAlloc(GMEM_SHARE | GMEM_ZEROINIT, sizeof(RWCTLINFO));
  if ( hInfo )
    {
    LPRWCTLINFO Info = (LPRWCTLINFO) GlobalLock(hInfo);
    Info->wVersion = 0x0100;             // Version 1.00 of DLL
    Info->wCtlTypes = 1;                 // Number of  controls in Function
    lstrcpy(Info->szClass, szClassName); // for instance of control
    lstrcpy(Info->szTitle, "Text");      // Caption
					 // fill control structure
    Info->Type[0].wType   = 0;           // Microsoft requires 0
    Info->Type[0].wWidth  = 0x8000 | 70; // default width
    Info->Type[0].wHeight = 0x8000 | 36; // default height
    lstrcpy(Info->Type[0].szDescr, "Font Text");//Must be different
    Info->Type[0].dwStyle = WS_VISIBLE | WS_CHILD ;
    Info->Type[0].hToolBit =            // Bitmap for Workshop palett
      LoadBitmap(CtlFntModule->hInstance, "CTLFNTBMP");
    Info->Type[0].hDropCurs =           // Cursor for Workshop
      LoadCursor(CtlFntModule->hInstance, "CTLFNTCUR");
    GlobalUnlock(hInfo);                // Unlock memory
    }
  return hInfo;                         // let RW have the memory
  }
//========================================================================
#pragma argsused            // Used by RW to define controls
extern "C" HANDLE FAR PASCAL _export ListClasses(LPSTR szAppName,
				      WORD wVersion,
				      LPFNLOADRES fnLoad,
				      LPFNEDITRES fnEdit)
{
    HANDLE hClasses = GlobalAlloc(GMEM_SHARE | GMEM_ZEROINIT,
	sizeof(int) + sizeof(RWCTLCLASS));
    if ( hClasses )
    {
	LPCTLCLASSLIST Classes = (LPCTLCLASSLIST) GlobalLock(hClasses);
	Classes->nClasses = 1;
	Classes->Classes[0].fnRWInfo  = FntTextInfo;  // Three functions
	Classes->Classes[0].fnRWStyle = FntTextStyle; // Workshop Calls
	Classes->Classes[0].fnFlags   = (LPFNFLAGS)FntTextFlags;
	lstrcpy(Classes->Classes[0].szClass, szClassName);
	GlobalUnlock(hClasses);
    }
    return hClasses;
}
//------------------------------------------------------------------------
WORD FAR PASCAL _export CTLFNTGetVersion(void)
  {
  return 0x0100;
  }
//------------------------------------------------------------------------
int FAR PASCAL _export CTLFNTAbout(void)
  {
  return MessageBox(GetFocus(),"By Bob Bourbonnais",
		    "Custom Control for Font Text",MB_OK);
  }
//------------------------------------------------------------------------
#pragma argsused
extern "C" int FAR PASCAL _export WEP(int nParameter)
{
  return (1);
}
//------------------------------------------------------------------------
#pragma argsused
extern "C" int FAR PASCAL LibMain(HINSTANCE hInstance, WORD wDataSeg,
				 WORD cbHeapSize, LPSTR lpCmdLine)
{
  if (cbHeapSize > 0)
    UnlockData(0);
  CtlFntModule = new TModule("FntTextMod", hInstance, lpCmdLine);
  RegisterFntTextClass(CtlFntModule->hInstance);
  return (1);
}
//=======================================================================//
